import type { FC } from 'react';
import type { IRoute } from 'umi';
import { useRef } from 'react';
import { useIntl, Link, useSelector, history } from 'umi';
//import Icon from '@ant-design/icons';
import { useMemoizedFn, useSafeState } from 'ahooks';
import ProLayout, { PageContainer } from '@ant-design/pro-layout';
import { HeaderLeft, HeaderRight } from './Header';
import styles from './layout.less';
import logo from '../assets/images/logo.png';
import { Image } from 'antd';
import '../styles/global.less';
import { ILinkCallBack, ILinkCallBackState } from '@/typings';

import { ConfigProvider } from 'antd';

import { IconMoon } from '@/components/IconMoon/IconMoon';

export const NoHeaderPaths: Dictionary<Boolean> = {
  '/home/order-detail/:id': true,
};

interface IProps {
  route: IRoute;
  location: any;
}

// 路由转换，将icon字符串转换为组件
const transformRoute = (route: IRoute) => {
  const { routes, icon } = route;
  if (icon && typeof icon === 'string') {
    route.svg = <IconMoon size={18} icon={icon} />;
    delete route.icon; // there is an issue of submenu not rendered
  }
  if (routes?.length) {
    routes.forEach((_route: IRoute) => {
      transformRoute(_route);
    });
  }
};

const BaseLayout: FC<IProps> = (props) => {
  const { route, children } = props;
  const intl = useIntl();
  const flag = useRef(false);
  const [collapsed, setCollapsed] = useSafeState(false);
  const leftHeaderRef = useRef<{ setTitle: (title: string) => void }>();
  const linkCallBacks = useSelector<ILinkCallBackState, ILinkCallBack>(
    (state) => state.linkCallback,
  );
  // const [currentSelectedMenu, setCurrentlySelectedMenu] = useSafeState<string>();

  if (!flag.current) {
    flag.current = true;
    transformRoute(route);
  }

  const title = intl.formatMessage({ id: 'menu.storeAdmin' });

  const renderRight = useMemoizedFn(({ matchMenuKeys }: any) => {
    return <HeaderRight matchMenuKeys={matchMenuKeys} />;
  });

  return (
    <ConfigProvider locale={{ locale: 'en' }}>
      <ProLayout
        fixSiderbar
        fixedHeader
        // @ts-ignore
        title={title}
        route={route}
        siderWidth={250}
        breadcrumbRender={false}
        // onPageChange={(e) => setCurrentlySelectedMenu(e?.pathname)}
        collapsed={collapsed}
        navTheme={'light'}
        logo={<Image width={62} height={'auto'} preview={false} src={logo} />}
        // 关闭collapsed
        collapsedButtonRender={false}
        rightContentRender={renderRight}
        // 国际化获取名称
        formatMessage={intl.formatMessage}
        // 这里不加Link无法跳转路由
        menuItemRender={(item, dom) => {
          const callback = Object.values(linkCallBacks)[0];
          return (
            <Link
              // @ts-ignore
              to={item.path}
              onClick={
                callback
                  ? async (e) => {
                      e.preventDefault();
                      const success = await callback();
                      if (success) history.push(item.path as string);
                    }
                  : undefined
              }
              className={styles.link}
            >
              {item.svg}
              {dom}
            </Link>
          );
        }}
        subMenuItemRender={(item, dom) => {
          return (
            // @ts-ignore
            <Link className={styles.link}>
              {item.svg}
              {dom}
            </Link>
          );
        }}
        // header 的自定义内容在这里渲染
        // header custom contain rendering
        headerContentRender={({ matchMenuKeys }: any) => {
          return (
            <HeaderLeft
              matchMenuKeys={matchMenuKeys}
              ref={leftHeaderRef}
              collapsed={collapsed}
              setCollapsed={setCollapsed}
            />
          );
        }}
        className={styles.global_layout}
      >
        <PageContainer
          pageHeaderRender={(params) => {
            const { matchMenuKeys } = params as any;
            const currentPath = matchMenuKeys[matchMenuKeys.length - 1];
            if (!NoHeaderPaths[currentPath])
              setTimeout(
                () => leftHeaderRef.current?.setTitle((params.title as string) || ''),
                0,
              );
            return null;
          }}
          className={styles.mainPageContainer}
        >
          {children}
        </PageContainer>
      </ProLayout>
    </ConfigProvider>
  );
};

export default BaseLayout;
